﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace WebMetar_NS
{
    /// <summary>
    /// General purpose Bar graph
    /// 
    /// Author: Dennis Lang  2010      
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    public partial class BarGraph : UserControl
    {
        public class DataItem
        {
            public DataItem()
            {
            }
            public DataItem(string _name, DateTime _dtUtc, bool _marked, double[] _values)
            {
                name = _name;
                dtUtc = _dtUtc;
                marked = _marked;
                values = _values;
            }

            public string name;
            public DateTime dtUtc;
            public bool marked;
            public double[] values;
        };

        List<DataItem> dataArray;
        string[] dataTip;
        double minVal, maxVal;
        int begDataIdx = 0;
        enum LabelField { eName, eTimeZ, eTimeL };
        LabelField labelField = LabelField.eName;

        [Serializable]
        public class BarGraphSettings
        {
            public BarGraphSettings()
            {
                vlColor = Pens.Wheat;
                hlColor = Pens.Tan;
                frameColor = Pens.Violet;

                bgBrush = new SolidBrush(Color.White);
                bgGrid = new SolidBrush(Color.LightGray);
                txBrush = new SolidBrush(Color.Blue);
                barBack = new SolidBrush(Color.Black);

                barLeftColors = new Color[] { Color.DarkBlue, Color.DarkGray, Color.Brown };
                barRightColors = new Color[] { Color.LightBlue, Color.LightGreen, Color.LightGray };
            }

            Font grFont;
            bool autoScale;
            float scale;
            float offset;

            [CategoryAttribute("General"),
            DescriptionAttribute("Label and Legend font")]
            public Font GrFont
            {
                get { return grFont; }
                set { grFont = value; }
            }

            [CategoryAttribute("General"),
            DescriptionAttribute("Automatic horizontal scaling")]
            public bool AutoScale
            {
                get { return autoScale; }
                set { autoScale = value; }
            }

            [CategoryAttribute("General"),
            DescriptionAttribute("Horizontal scaling")]
            public float Scale
            {
                get { return scale; }
                set { scale = value; }
            }

            [CategoryAttribute("General"),
            DescriptionAttribute("Horizontal offset")]
            public float Offset
            {
                get { return offset; }
                set { offset = value; }
            }


            [NonSerialized]
            Pen vlColor,
                hlColor,
                frameColor;

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Vertical Line Color")]
            public Pen VlColor
            {
                get { return vlColor; }
                set { vlColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Vertical Line Color")]
            public Color VlPenColor
            {
                get { return vlColor.Color; }
                set { vlColor = new Pen(value, vlColor.Width); }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Horizontal Line Color")]
            public Pen HlColor
            {
                get { return hlColor; }
                set { hlColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Horizontal Line Color")]
            public Color HlPenColor
            {
                get { return hlColor.Color; }
                set { hlColor = new Pen(value, hlColor.Width); }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Frame Line Color")]
            public Pen FrameColor
            {
                get { return frameColor; }
                set { frameColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Frame Line Color")]
            public Color FramePenColor
            {
                get { return frameColor.Color; }
                set { frameColor = new Pen(value, frameColor.Width); }
            }

            [NonSerialized]
            SolidBrush bgBrush,
                bgGrid,
                txBrush,
                barBack,
                markBrush;
            [NonSerialized]

            Color[] barLeftColors;
            Color[] barRightColors;

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Legend background Color")]
            public SolidBrush BgBrush
            {
                get { return bgBrush; }
                set { bgBrush = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Legend background Color")]
            public Color BgLegendColor
            {
                get { return bgBrush.Color; }
                set { bgBrush.Color = value; }
            }

            [CategoryAttribute("Colors"),
             Browsable(false),
             DescriptionAttribute("Graph background Color")]
            public SolidBrush BgGrid
            {
                get { return bgGrid; }
                set { bgGrid = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Graph background Color")]
            public Color BgGridColor
            {
                get { return bgGrid.Color; }
                set { bgGrid.Color = value; }
            }

            [CategoryAttribute("Colors"),
             Browsable(false),
             DescriptionAttribute("Text foreground Color")]
            public SolidBrush TxBrush
            {
                get { return txBrush; }
                set { txBrush = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Text foreground Color")]
            public Color TextFgColor
            {
                get { return txBrush.Color; }
                set { txBrush.Color = value; }
            }

            [CategoryAttribute("Colors"),
            DescriptionAttribute("Bar Left Color(s)")]
            public Color[] BarLeftColors
            {
                get { return barLeftColors; }
                set  { barLeftColors = value; }
            }

            [CategoryAttribute("Colors"),
            DescriptionAttribute("Bar Right Color(s)")]
            public Color[] BarRightColors
            {
                get { return barRightColors; }
                set { barRightColors = value; }
            }

            [CategoryAttribute("Colors"),
            DescriptionAttribute("Bar background Color")]
            public Color BarBackColor
            {
                get { return barBack.Color; }
                set { barBack.Color = value; }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Bar background Color")]
            public SolidBrush BarBack
            {
                get { return barBack; }
                set { barBack = value; }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Bar mark coloor")]
            public SolidBrush MarkBrush
            {
                get { return markBrush; }
                set { markBrush = value; }
            }

            Size cellSize;
            Size barSize;
            Size shadowSize;
            Size barMargin;
            Size ulMargin;
            Size lrMargin;

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Cell Size")]
            public Size CellSize
            {
                get { return cellSize; }
                set { cellSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Size")]
            public Size BarSize
            {
                get { return barSize; }
                set { barSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Shadow")]
            public Size ShadowSize
            {
                get { return shadowSize; }
                set { shadowSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Margin Size")]
            public Size BarMargin
            {
                get { return barMargin; }
                set { barMargin = value; }
            }

            [CategoryAttribute("Sizes"),
             DescriptionAttribute("UpperLeft Margin")]
            public Size UlMargin
            {
                get { return ulMargin; }
                set { ulMargin = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("LowerRight Margin")]
            public Size LrMargin
            {
                get { return lrMargin; }
                set { lrMargin = value; }
            }

            Size titleSize;
            string hTitle;

            [CategoryAttribute("Text"),
            DescriptionAttribute("Title Size")]
            public Size TitleSize
            {
                get { return titleSize; }
                set { titleSize = value; }
            }

            [CategoryAttribute("Text"),
            DescriptionAttribute("Horizontal Title")]
            public string HTitle
            {
                get { return hTitle; }
                set { hTitle = value; }
            }
           
        };

        BarGraphSettings s;

        public object Settings
        {
            get { return s; }
            set { s = value as BarGraphSettings;}
        }

        public BarGraphSettings BarSettings
        {
            get { return s; }
            set { s = value; UpdateInternalSettings();  }
        }

        const UInt64 one = 1;

        public BarGraph()
        {
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.DoubleBuffer, true);

            DefaultSettings();
            UpdateInternalSettings();
            InitializeComponent();

            KeyUp += new KeyEventHandler(BarGraph_KeyUp);
        }

        void BarGraph_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Up:
                    if (begDataIdx > 0)
                    {
                        begDataIdx--;
                        Refresh();
                    }
                    break;
                case Keys.Down:
                    if (begDataIdx + 1 < dataArray.Count)
                    {
                        begDataIdx++;
                        Refresh();
                    }
                    break;
                case Keys.Left:
                    labelField = (LabelField)(((int)labelField - 1 + 3) % 3);
                    Refresh();
                    break;            
                case Keys.Right:
                    labelField = (LabelField)(((int)labelField + 1) % 3);
                    Refresh();
                    break;
            }
        }

        public void DefaultSettings()
        {
            s = new BarGraphSettings();

            s.AutoScale = true;
            s.Scale = 1.0f;
            s.Offset = 0.0f;

            s.VlColor = Pens.Wheat;
            s.HlColor = Pens.Tan;
            s.FrameColor = Pens.Violet;

            s.BgBrush = new SolidBrush(Color.White);
            s.BgGrid = new SolidBrush(Color.LightGray);
            s.TxBrush = new SolidBrush(Color.Blue);
            s.BarBack = new SolidBrush(Color.DarkGray);
            s.MarkBrush = new SolidBrush(Color.Red);

            s.CellSize = new Size(50, 50);
            s.BarSize = new Size(10, 10);
            s.ShadowSize = new Size(4, 4);
            s.BarMargin = new Size(4, 4);

            s.GrFont = new Font(this.Font.FontFamily, this.Font.Size, FontStyle.Bold);
            int fontHeight = s.GrFont.Height;
            s.TitleSize = new Size(0, fontHeight + 2);
            s.UlMargin = new Size(40, FontHeight + 2);
            s.LrMargin = s.UlMargin;
            s.UlMargin += s.TitleSize;

            s.BarLeftColors = new Color[] { Color.DarkBlue, Color.DarkGray, Color.Brown };
            s.BarRightColors = new Color[] { Color.LightBlue, Color.LightGreen, Color.LightGray };

            s.HTitle = "Temperature F";
            List<DataItem> demoData = new List<DataItem>();
            DateTime now = DateTime.UtcNow;

            demoData.Add(new DataItem("KBOS", now, false, new double[] { 32.0, 33.0, 10.0 }));
            demoData.Add(new DataItem("KMIA", now, true, new double[] { 34.0, 43.0, 50.0 }));
            demoData.Add(new DataItem("KDFW", now, false, new double[] { 36.0, 23.0, 15.0 }));
            demoData.Add(new DataItem("KRDG", now, false, new double[] { 12.0, 13.0, 19.0 }));

            string[] tip = new string[] { "{1} Temperature {0:F1} ", "{1} DewPoint {0:F1}",  "{1} WindSpeed {0:F1}", " bogus2 "};
            Data(demoData, tip);
        }

        public void Clear()
        {
            this.dataArray = null;
            this.Refresh();
        }

        [NonSerialized]
        Brush[] barBrushes;

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            this.UpdateInternalSettings();
        }

        public void UpdateInternalSettings()
        {
            if (s != null && s.BarLeftColors != null && 
                s.BarLeftColors.Length > 0 &&
                s.BarLeftColors.Length <= s.BarRightColors.Length)
            {
                barBrushes = new LinearGradientBrush[s.BarLeftColors.Length];
                Point lr = new Point(this.Width, this.Height);
                for (int barIdx = 0; barIdx < s.BarLeftColors.Length; barIdx++)
                {
                    barBrushes[barIdx] = new LinearGradientBrush(Point.Empty, lr,
                        s.BarLeftColors[barIdx], s.BarRightColors[barIdx]);
                }
            }
        }

        public void Data(List<DataItem> _dataArray, string[] _tip)
        {
            dataArray = _dataArray;
            if (_tip != null)
                dataTip = _tip;

            minVal = double.MaxValue;
            maxVal = double.MinValue;

            if (dataArray == null || dataArray.Count == 0)
                return;

            int dataCnt = dataArray.Count;
            begDataIdx = Math.Min(begDataIdx, dataCnt - 1);
            for (int dataIdx = begDataIdx; dataIdx < dataCnt; dataIdx++)
            {
                DataItem di = dataArray[dataIdx];

                for (int valueIdx = 0; valueIdx < di.values.Length; valueIdx++)
                {
                    double val1 = di.values[valueIdx];
                    minVal = Math.Min(minVal, val1);
                    maxVal = Math.Max(maxVal, val1);
                }
            }

            if (s.AutoScale)
            {
                s.Offset = 0;
                if (this.minVal < 0)
                {
                    s.Offset = ((int)(this.minVal / 10) - 2) * 10;
                }

                int n10 = 1;
                double tmpMax = this.maxVal - s.Offset;
                while (tmpMax >= 10)
                {
                    n10 *= 10;
                    tmpMax /= 10;
                }

                s.Scale = n10;
                while (tmpMax > 0)
                {
                    s.Scale += n10;
                    tmpMax -= 1;
                }

               
            }

            this.Refresh();
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            if (s == null || s.CellSize.Width == 0 || s.BgBrush == null)
                DefaultSettings();

            // base.OnPaintBackground(e);
            
            // TODO - draw negative left side with different background color
            // than right positive side when s.Offset < 0
            e.Graphics.FillRectangle(s.BgBrush, this.DisplayRectangle);

            Size dspSize = DisplayRectangle.Size;
            Size grSize = dspSize - s.UlMargin - s.LrMargin;
            float fontMidHeight = s.GrFont.Height / 2.0f;

            if (this.dataArray != null)
            {
                for (int dataIdx = begDataIdx; dataIdx < this.dataArray.Count; dataIdx++)
                {
                    int rowIdx = dataIdx - begDataIdx;
                    DataItem di = dataArray[dataIdx];
                    string legend_hyphen = string.Empty;
                    switch (labelField)
                    {
                        case LabelField.eName:
                            legend_hyphen = di.name + "-";
                            break;
                        case LabelField.eTimeZ:
                            legend_hyphen = di.dtUtc.ToString("HH:mm") + "z-";
                            break;
                        case LabelField.eTimeL:
                            {
                                DateTime localDt = di.dtUtc.Add(TimeZone.CurrentTimeZone.GetUtcOffset(DateTime.Now));
                                legend_hyphen = localDt.ToString(" HH:mm") + "-";
                                break;
                            }
                    }
                    SizeF legendSize = e.Graphics.MeasureString(legend_hyphen, s.GrFont);

                    float col = s.UlMargin.Width - legendSize.Width - 1;
                    float row = s.CellSize.Height * (rowIdx + 0.5f) + s.UlMargin.Height - fontMidHeight;

                    e.Graphics.DrawString(legend_hyphen, s.GrFont, s.TxBrush, col, row);
                }
            }

            // Title 
            SizeF hTitleSize = e.Graphics.MeasureString(s.HTitle, s.GrFont);
            e.Graphics.DrawString(s.HTitle, s.GrFont, s.TxBrush, s.UlMargin.Width + (grSize.Width - hTitleSize.Width) / 2, 0);

            // Column headings.
            float lblRow = s.TitleSize.Height;
            for (float lblCol = s.CellSize.Width; lblCol <= grSize.Width; lblCol += s.CellSize.Width)
            {
                double val = lblCol / (float)grSize.Width * s.Scale + s.Offset;
                string label = val.ToString("F0");
                SizeF labelSize = e.Graphics.MeasureString(label, s.GrFont);
                e.Graphics.DrawString(label, s.GrFont, s.TxBrush, lblCol + s.UlMargin.Width - labelSize.Width / 2, lblRow);
            }
            
            e.Graphics.TranslateTransform(s.UlMargin.Width, s.UlMargin.Height);
            e.Graphics.FillRectangle(s.BgGrid, 0, 0, grSize.Width, grSize.Height);

            if (this.dataArray != null)
            {
                float barOff = s.BarSize.Height + s.BarMargin.Height / 2;

                for (int dataIdx = begDataIdx; dataIdx < this.dataArray.Count; dataIdx++)
                {
                    int rowIdx = dataIdx - begDataIdx;
                    DataItem di = this.dataArray[dataIdx];
                    float topRow = s.CellSize.Height * rowIdx + s.BarMargin.Height / 2;

                    if (s.ShadowSize.Width > 0 || s.ShadowSize.Height > 0)
                    {
                        for (int valueIdx = 0; valueIdx < di.values.Length; valueIdx++)
                        {
                            double val = di.values[valueIdx];
                            // per = (rawValue - offset) / scale
                            float per = (float)((val - s.Offset) / s.Scale);
                            float row = topRow + barOff * valueIdx;

                            Brush shadowBrush = di.marked ? s.MarkBrush : s.BarBack;
                            e.Graphics.FillRectangle(shadowBrush, 0, row + s.ShadowSize.Height,
                                grSize.Width * per + s.ShadowSize.Width, s.BarSize.Height);
                        }
                    }

                    int maxBrushes = barBrushes.Length;
                    for (int valueIdx = 0; valueIdx < di.values.Length && maxBrushes > 0; valueIdx++)
                    {
                        double val = di.values[valueIdx];
                        float per = (float)((val - s.Offset) / s.Scale);
                        float row = topRow + barOff * valueIdx;

                        e.Graphics.FillRectangle(barBrushes[valueIdx % maxBrushes], 
                                0, row, grSize.Width * per, s.BarSize.Height);
                    }
                } 
            }

            for (int vColumn = 0; vColumn < grSize.Width; vColumn += s.CellSize.Width)
            {
                e.Graphics.DrawLine(s.VlColor, vColumn, 0, vColumn, grSize.Height);
                e.Graphics.DrawLine(Pens.Black, vColumn + 1, 0, vColumn + 1, grSize.Height);
            }
            for (int hRow = 0; hRow < grSize.Height; hRow += s.CellSize.Height)
            {
                e.Graphics.DrawLine(s.HlColor, 0, hRow, grSize.Width, hRow);
                e.Graphics.DrawLine(Pens.Green, 0, hRow + 1, grSize.Width, hRow + 1);
            }
            
            e.Graphics.DrawRectangle(s.FrameColor, 0, 0, grSize.Width, grSize.Height);
            e.Graphics.TranslateTransform(-s.UlMargin.Width, -s.UlMargin.Height);
        }

        protected override void OnPrint(PaintEventArgs e)
        {
            base.OnPrint(e);
        }

        /// http://www.codeproject.com/KB/printing/printawinformsusercontrol.aspx
        /// <summary>
        /// Print the control's view to a Graphics object.
        /// </summary>
        /// <param name="graphics">Graphics object to draw on.</param>
        /// <param name="bounds">Rectangle to print in.</param>
        public void PrintToGraphics(Graphics graphics, RectangleF bounds)
        {
            Bitmap bitmap = new Bitmap(this.Width, this.Height);
            this.DrawToBitmap(bitmap, new Rectangle(0, 0, bitmap.Width, bitmap.Height));
            RectangleF target = bounds; //  new RectangleF(0, 0, bounds.Width, bounds.Height);
            double xScale = (double)bitmap.Width / bounds.Width;
            double yScale = (double)bitmap.Height / bounds.Height;
            if (xScale < yScale)
                target.Width = (int)(xScale * target.Width / yScale);
            else
                target.Height = (int)(yScale * target.Height / xScale);
            graphics.PageUnit = GraphicsUnit.Display;
            graphics.DrawImage(bitmap, target);
        }

        private void BarGraph_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.dataArray == null || this.dataArray.Count == 0 || s.CellSize.Width == 0)
                return;

            Size mouseLoc = new Size(e.Location.X, e.Location.Y);
            mouseLoc -= s.UlMargin;
            if (mouseLoc.Width > 0 && mouseLoc.Height > 0)
            {
                int rowIdx = (mouseLoc.Height / s.CellSize.Height);
                int dataIdx = rowIdx + begDataIdx;
                if (dataIdx < this.dataArray.Count)
                {
                    DataItem di = this.dataArray[dataIdx];

                    int subHeight = s.CellSize.Height / di.values.Length;
                    int subIdx = (mouseLoc.Height - rowIdx * s.CellSize.Height) / subHeight;
                    subIdx = Math.Min(subIdx, di.values.Length - 1);

                    // System.Diagnostics.Debug.WriteLine(testNum.ToString()+  " " + subTestNum.ToString());

                    float x = e.Location.X + 20;
                    float y = e.Location.Y + 20;
               //     float y = dataIdx * s.CellSize.Height + s.UlMargin.Height + (subIdx + 0.5f) * subIdx;
                 
                    string msg = string.Format(dataTip[subIdx],
                        di.values[subIdx], di.name, di.dtUtc);
                    this.toolTip.Show(msg, this, (int)x, (int)y, 5000);
                }
            }
        }
    }
}
