﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WebMetar_NS
{
    /// <summary>
    /// Helper class to extract text from HTML page.
    /// 
    /// Author: Dennis Lang - 2010
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// </summary>
    class HtmlUtil
    {
        static string[] escapeStr  = new string[] {
            "&nbsp;",
            "&quot;",
            "&amp;",
            "&lt;",
            "&gt;"} ;
        static string[] valueStr = new string[] {" ", "\"", "&", "<", ">" };

        /// <summary>
        /// Extract text from html page.
        /// </summary>
        /// <param name="html">html input</param>
        /// <returns>raw text</returns>
        static public string HtmlToText(string html)
        {
            string text = string.Empty;
            bool xmlBeg = false;
            string xmlTag = string.Empty;

            for (int pos = 0; pos < html.Length; pos++)
            {
                char c = html[pos];

                if (xmlBeg)
                {
                    if (c == '>')
                    {
                        xmlBeg = false;
                        switch (xmlTag.ToLower().Trim())
                        {
                            case "/table":
                            case "tr":
                            case "p":
                            case "br":
                            case "li":
                                text += "\n";
                                break;
                            case "td":
                                text += " ";
                                break;
                        }
                    }
                }
                else if (c == '<')
                {
                    xmlBeg = true;
                    xmlTag = html.Substring(pos + 1, XmlLength(html, pos + 1));
                }
                else if (c == '&')
                {
                    bool matched = false;
                    for (int escIdx = 0; escIdx < escapeStr.Length; escIdx++)
                    {
                        string token = escapeStr[escIdx];
                        if (html.IndexOf(token, pos, token.Length, StringComparison.OrdinalIgnoreCase) != -1)
                        {
                            text += valueStr[escIdx];
                            matched = true;
                            break;
                        }
                    }

                    if (false == matched)
                        text += c;
                }
                else
                {
                    if (char.IsControl(c))
                        text += ' ';
                    else
                        text += c;
                }
            }

            return text;
        }

        static public int XmlLength(string html, int pos)
        {
            int beg = pos;

            for ( ; pos < html.Length; pos++)
            {
                if (html[pos] == '>' || char.IsWhiteSpace(html[pos]))
                    return pos - beg;
            }
            return pos - beg;
        }
    }
}
