﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;


namespace WebMetar_NS
{
    /// <summary>
    /// Regular expression wrapper class to make it easier to get at associated matching text groups.
    /// 
    /// * IsMatch returns true if regular expression matches (without ^ or $ will find match anywhere)
    /// * IsMatches returns true and saves all associated text match groups. See 'groups' and 'indexes'
    ///   
    /// Author: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    class RegCompare
    {
        public RegCompare(string regPattern)
        {
            m_regex = new Regex(regPattern, RegexOptions.IgnoreCase);
        }

        Regex m_regex;
        Match m_match;

        /// <summary>
        /// Associated matching text groups. groups[0] is full text, [1]=first group, etc.
        /// </summary>
        public string[] groups;
        /// <summary>
        /// Starting position of  matching input text by group. [0] is full text.
        /// </summary>
        public int[] indexes;

        public bool IsMatch(string text)
        {
            return m_regex.IsMatch(text);
        }

        public bool IsMatch(string text, int startAt)
        {
            return m_regex.IsMatch(text, startAt);
        }

        public bool IsMatches(string text)
        {
            return IsMatches(text, 0);
        }

        public bool IsMatches(string text, int startAt)
        {
            if (text == "")
                return false;

            bool isMatch;
            if (isMatch = m_regex.IsMatch(text, startAt))
            {
                m_match = m_regex.Match(text, startAt);
                groups = new string[m_match.Groups.Count];
                indexes = new int[m_match.Groups.Count];
                for (int grpIdx = 0; grpIdx < m_match.Groups.Count; grpIdx++)
                {
                    groups[grpIdx] = (m_match.Groups[grpIdx].Captures.Count > 0) ?
                        m_match.Groups[grpIdx].Captures[0].Value : string.Empty;
                    indexes[grpIdx] = m_match.Groups[grpIdx].Index;
                }
            }

            return isMatch;
        }
    }
}
