﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace WebMetar_NS
{
    /// <summary>
    /// Time series graph (x-axis spans 1 hour, y-axis span 'n' hours)
    /// 
    /// Author: Dennis Lang  2010      
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    public partial class TimeGraph : UserControl
    {
        public class DataItem
        {
            public DataItem()
            {
            }

            public DateTime dt;
            public string name;
        };

        DateTime baseDT = DateTime.Now;
        List<DataItem> dataArray;
        List<string> labels;
        string[] dataTip;
        DateTime minVal, maxVal;

        [Serializable]
        public class TimeGraphSettings
        {
            public TimeGraphSettings()
            {
                vlColor = Pens.Wheat;
                hlColor = Pens.Tan;
                frameColor = Pens.Violet;

                bgGridAlpha = 64;
                bgGrid = new SolidBrush(Color.LightGray);
                bgBrush = new SolidBrush(Color.White);
                txBrush = new SolidBrush(Color.Blue);
                symBack = new SolidBrush(Color.Black);

                Point left = new Point(0, 0);
                Point right = new Point(600, 0);

                cellBegColors = new Color[] { Color.Red, Color.Green, Color.Blue };
                cellEndColors = new Color[] { Color.White, Color.White, Color.White };
            }

            Font grFont;
            bool autoScale;
            float hours;

            [CategoryAttribute("General"),
            DescriptionAttribute("Label and Legend font")]
            public Font GrFont
            {
                get { return grFont; }
                set { grFont = value; }
            }

            [CategoryAttribute("General"),
            DescriptionAttribute("Automatic scaling")]
            public bool AutoScale
            {
                get { return autoScale; }
                set { autoScale = value; }
            }

            [CategoryAttribute("General"),
            DescriptionAttribute("Hours")]
            public float Hours
            {
                get { return hours; }
                set { hours = value; }
            }

            [NonSerialized]
            Pen vlColor,
                hlColor,
                frameColor;

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Vertical Line Color")]
            public Pen VlColor
            {
                get { return vlColor; }
                set { vlColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Vertical Line Color")]
            public Color VlPenColor
            {
                get { return vlColor.Color; }
                set { vlColor = new Pen(value, vlColor.Width); }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Horizontal Line Color")]
            public Pen HlColor
            {
                get { return hlColor; }
                set { hlColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Horizontal Line Color")]
            public Color HlPenColor
            {
                get { return hlColor.Color; }
                set { hlColor = new Pen(value, hlColor.Width); }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Frame Line Color")]
            public Pen FrameColor
            {
                get { return frameColor; }
                set { frameColor = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Frame Line Color")]
            public Color FramePenColor
            {
                get { return frameColor.Color; }
                set { frameColor = new Pen(value, frameColor.Width); }
            }

            Color[] cellBegColors;
            Color[] cellEndColors;
            int bgGridAlpha;

            [NonSerialized]
            SolidBrush bgBrush,
                bgGrid,
                txBrush,
                symBack;

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Legend background Color")]
            public SolidBrush BgBrush
            {
                get { return bgBrush; }
                set { bgBrush = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Legend background Color")]
            public Color BgLegendColor
            {
                get { return bgBrush.Color; }
                set { bgBrush.Color = value; }
            }

            [CategoryAttribute("Colors"),
             Browsable(false),
             DescriptionAttribute("Graph background Color")]
            public SolidBrush BgGrid
            {
                get { return bgGrid; }
                set { bgGrid = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Graph background Color")]
            public Color BgGridColor
            {
                get { return bgGrid.Color; }
                set { bgGrid.Color = value; }
            }
            [CategoryAttribute("Colors"),
           DescriptionAttribute("Graph background Alpha")]
            public int BgGridAlpha
            {
                get { return bgGridAlpha; }
                set { bgGridAlpha = value; }
            }

            [CategoryAttribute("Colors"),
             Browsable(false),
             DescriptionAttribute("Text foreground Color")]
            public SolidBrush TxBrush
            {
                get { return txBrush; }
                set { txBrush = value; }
            }
            [CategoryAttribute("Colors"),
            DescriptionAttribute("Text foreground Color")]
            public Color TextFgColor
            {
                get { return txBrush.Color; }
                set { txBrush.Color = value; }
            }

            [CategoryAttribute("Colors"),
             DescriptionAttribute("Cell Begin Array")]
            public Color[] CellBeginColors
            {
                get { return cellBegColors; }
                set { cellBegColors = value; }
            }

            [CategoryAttribute("Colors"),
            DescriptionAttribute("Cell End Array")]
            public Color[] CellEndColors
            {
                get { return cellEndColors; }
                set { cellEndColors = value; }
            }

            [CategoryAttribute("Colors"),
            DescriptionAttribute("Symbol background Color")]
            public Color SymBackColor
            {
                get { return symBack.Color; }
                set { symBack.Color = value; }
            }

            [CategoryAttribute("Colors"),
            Browsable(false),
            DescriptionAttribute("Symbol background Color")]
            public SolidBrush SymBack
            {
                get { return symBack; }
                set { symBack = value; }
            }

            Size cellSize;
            Size symSize;
            Size shadowSize;
            Size symMargin;
            Size ulMargin;
            Size lrMargin;

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Cell Size")]
            public Size CellSize
            {
                get { return cellSize; }
                set { cellSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Size")]
            public Size SymSize
            {
                get { return symSize; }
                set { symSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Shadow")]
            public Size ShadowSize
            {
                get { return shadowSize; }
                set { shadowSize = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("Bar Margin Size")]
            public Size SymMargin
            {
                get { return symMargin; }
                set { symMargin = value; }
            }

            [CategoryAttribute("Sizes"),
             DescriptionAttribute("UpperLeft Margin")]
            public Size UlMargin
            {
                get { return ulMargin; }
                set { ulMargin = value; }
            }

            [CategoryAttribute("Sizes"),
            DescriptionAttribute("LowerRight Margin")]
            public Size LrMargin
            {
                get { return lrMargin; }
                set { lrMargin = value; }
            }

            Size titleSize;
            double hScale;
            double vScale;
            string hTitle;
            string vTitle;

            [CategoryAttribute("Text"),
            DescriptionAttribute("Title Size")]
            public Size TitleSize
            {
                get { return titleSize; }
                set { titleSize = value; }
            }

            [CategoryAttribute("Text"),
            DescriptionAttribute("Horizontal Scale")]
            public double HScale
            {
                get { return hScale; }
                set { hScale = value; }
            }

            [CategoryAttribute("Text"),
            DescriptionAttribute("Vertical Scale")]
            public double VScale
            {
                get { return vScale; }
                set { vScale = value; }
            }

            [CategoryAttribute("Text"),
            DescriptionAttribute("Horizontal Title")]
            public string HTitle
            {
                get { return hTitle; }
                set { hTitle = value; }
            }

            [CategoryAttribute("Text"),
            DescriptionAttribute("Vertical Scale")]
            public string VTitle
            {
                get { return vTitle; }
                set { vTitle = value; }
            }
        };

        TimeGraphSettings s;

        public object Settings
        {
            get { return s; }
            set { s = value as TimeGraphSettings;}
        }

        public TimeGraphSettings BarSettings
        {
            get { return s; }
            set { s = value; }
        }

        const UInt64 one = 1;

        public TimeGraph()
        {
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.DoubleBuffer, true);

            DefaultSettings();
            InitializeComponent();
        }

        public void DefaultSettings()
        {
            s = new TimeGraphSettings();

            s.AutoScale = true;

            s.VlColor = Pens.Wheat;
            s.HlColor = Pens.Tan;
            s.FrameColor = Pens.Violet;

            s.BgGridAlpha = 180;
            s.BgGrid = new SolidBrush(Color.LightGray);
            s.BgBrush = new SolidBrush(Color.White);
            s.TxBrush = new SolidBrush(Color.Blue);
            s.SymBack = new SolidBrush(Color.DarkGray);

            s.SymSize = new Size(10, 10);
            s.ShadowSize = new Size(4, 4);
            s.SymMargin = new Size(4, 4);
            s.CellSize = s.SymSize + s.ShadowSize + s.SymMargin;

            s.GrFont = new Font(this.Font.FontFamily, this.Font.Size, FontStyle.Bold);
            int fontHeight = s.GrFont.Height;
            s.TitleSize = new Size(0, fontHeight + 2);
            s.UlMargin = new Size(40, FontHeight + 2);
            s.LrMargin = new Size(4, (FontHeight + 2) * 3);   // assume max 3 rows of legend
            s.UlMargin += s.TitleSize;

            Point left = new Point(0, 0);
            Point right = new Point(600, 0);
            s.HScale = 1.0;

            s.HTitle = "Time Graph";

            string[] tip = new string[] { "{1} Time {0:F1} " };

            string icao1 = "KBOS";
            string icao2 = "KMIA";
            string icao3 = "KDFW";
            List<DataItem> testData = new List<DataItem>();
            List<string> stations = new List<string>();
            stations.Add(icao1);
            stations.Add(icao2);
            stations.Add(icao3);

            testData.Add(new DataItem() { dt = DateTime.Now + new TimeSpan(0,  0, 0), name=icao1}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(0,  5, 0), name=icao2}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(1, 10, 0), name=icao3}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(2, 15, 0), name=icao1}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(3, 20, 0), name=icao2}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(4, 25, 0), name=icao3}); 
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(5, 30, 0), name=icao1});
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(6, 35, 0), name=icao2});
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(7, 40, 0), name=icao3});
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(8, 45, 0), name=icao1});
            testData.Add(    new DataItem(){dt=DateTime.Now + new TimeSpan(9, 50, 0), name=icao2});

            Data(testData, DateTime.Now, stations, tip);
        }

        public void Clear()
        {
            this.dataArray = null;
            this.Refresh();
        }

        public void Data(List<DataItem> _dataArray, DateTime _baseDT, List<string> _labels, string[] _tip)
        {
            dataArray = _dataArray;
            labels = _labels;
            baseDT = _baseDT;

            // Truncate to hour
            baseDT = baseDT.AddMinutes(- baseDT.Minute);
            baseDT = baseDT.AddSeconds(- baseDT.Second);

            if (_tip != null)
                dataTip = _tip;

            minVal = DateTime.MaxValue;
            maxVal = DateTime.MinValue;

            if (dataArray == null || dataArray.Count == 0)
                return;

            int numItems = dataArray.Count;
            for (int itemIdx = 0; itemIdx < numItems; itemIdx++)
            {
                DataItem di = dataArray[itemIdx];
                minVal = (minVal < di.dt) ? minVal : di.dt;
                maxVal = (maxVal > di.dt) ? maxVal : di.dt;
            }

            if (s.AutoScale)
            {
                s.Hours = (int)(maxVal - minVal).TotalHours;
                if (s.Hours < 0)
                    s.Hours = 1;
            }

            this.Refresh();
        }

        private void DrawShape(Graphics graphics, Brush brush, PointF pt, int shpIdx)
        {
            Point[] triPoints = new Point[] { 
                    new Point(0, -s.SymSize.Height/2),
                    new Point(s.SymSize.Width/2, s.SymSize.Height/2),
                    new Point(-s.SymSize.Width/2, s.SymSize.Height/2),
                    new Point(0, -s.SymSize.Height/2)};

            switch (shpIdx)
            {
                case 0:
                    graphics.FillEllipse(brush, new RectangleF(pt, s.SymSize));
                    break;

                case 1:
                    graphics.TranslateTransform(pt.X, pt.Y);
                    graphics.FillPolygon(brush, triPoints);
                    graphics.TranslateTransform(-pt.X, -pt.Y);
                    break;

                case 2:
                    graphics.DrawEllipse(new Pen(brush, 2), new RectangleF(pt, s.SymSize));
                    break;

                case 3:
                    graphics.TranslateTransform(pt.X, pt.Y);
                    graphics.DrawPolygon(new Pen(brush, 2), triPoints);
                    graphics.TranslateTransform(-pt.X, -pt.Y);
                    break;

                default:
                    graphics.FillRectangle(brush, new RectangleF(pt, s.SymSize));
                    break;
            }
        }

        Image bgImage = null;

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            AdjustSettings();

            // Tweak background image.
            if (this.BackgroundImage != null && this.Size.Height > 10 && s != null && s.BgGridAlpha < 200)
            {
                bgImage = new Bitmap(this.BackgroundImage, this.Size);
            }
            else
            {
                bgImage = null;
            }
        }

        protected override void OnVisibleChanged(EventArgs e)
        {
            base.OnVisibleChanged(e);
            AdjustSettings();
        }

        private void AdjustSettings()
        {
            if (s != null)
            {
                Size dspSize = DisplayRectangle.Size;
                Size grSize = dspSize - s.UlMargin - s.LrMargin;
                Size cellSize = new Size(grSize.Width / 12, s.CellSize.Height);
                s.CellSize = cellSize;
            }
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // base.OnPaintBackground(e);

            if (s == null || s.CellSize.Width == 0 || s.BgBrush == null)
                DefaultSettings();

            if (this.DisplayRectangle.Height < 10 || this.DisplayRectangle.Width < 10)
                return;

            if (this.bgImage != null)
            {
                e.Graphics.DrawImage(bgImage, Point.Empty);
            }
            else
            {
                e.Graphics.FillRectangle(s.BgBrush, this.DisplayRectangle);
            }

            Size dspSize = DisplayRectangle.Size;
            Size grSize = dspSize - s.UlMargin - s.LrMargin;
            // Size cellSize = new Size(grSize.Width / 12, s.CellSize.Height);
            grSize.Width = s.CellSize.Width * 12;
            int minInc = 60 / (grSize.Width / s.CellSize.Width  -1);

            {
                // Vertical Hour labels
                float fontMidHeight = s.GrFont.Height / 2.0f;
                int numHourRows = grSize.Height / s.CellSize.Height;
                TimeSpan oneHour = new TimeSpan(1, 0, 0);
                DateTime rowDT = baseDT;
                for (int hour = 0; hour < numHourRows; hour++, rowDT += oneHour)
                {
                    string legend_hyphen = rowDT.ToString("HH") + "-";
                    SizeF legendSize = e.Graphics.MeasureString(legend_hyphen, s.GrFont);

                    float col = s.UlMargin.Width - legendSize.Width - 1;
                    float row = s.CellSize.Height * (hour + 1) + s.UlMargin.Height - fontMidHeight;

                    e.Graphics.DrawString(legend_hyphen, s.GrFont, s.TxBrush, col, row);
                }

                // Title 
                SizeF hTitleSize = e.Graphics.MeasureString(s.HTitle, s.GrFont);
                e.Graphics.DrawString(s.HTitle, s.GrFont, s.TxBrush, s.UlMargin.Width + (grSize.Width - hTitleSize.Width) / 2, 0);

                // Horizontal Column minute headings.
                float lblRow = s.TitleSize.Height;
                int min = minInc;
                for (float lblCol = s.CellSize.Width; lblCol <= grSize.Width; lblCol += s.CellSize.Width, min += minInc)
                {
                    string label = min.ToString("#0");
                    SizeF labelSize = e.Graphics.MeasureString(label, s.GrFont);
                    e.Graphics.DrawString(label, s.GrFont, s.TxBrush, lblCol + s.UlMargin.Width - labelSize.Width / 2, lblRow);
                }

                // Bottom legend
                int lblIdx = 0;
                int legendRow = grSize.Height + s.UlMargin.Height;
                int legendCol = s.UlMargin.Width;
                foreach (string label in labels)
                {
                    int clrIdx = lblIdx % s.CellBeginColors.Length;
                    int shpIdx = lblIdx / s.CellBeginColors.Length;
                    SolidBrush brush = new SolidBrush(s.CellBeginColors[clrIdx]);
                    
                    SizeF labelSize = e.Graphics.MeasureString(label, s.GrFont);
                    int legendWidth = (int)(s.UlMargin.Width + labelSize.Width + s.CellSize.Width);
                    if (legendCol + legendWidth > s.UlMargin.Width + grSize.Width)
                    {
                        legendCol = s.UlMargin.Width;
                        legendRow += (int)(labelSize.Height * 1.2);
                    }

                    e.Graphics.DrawString(label, s.GrFont, s.TxBrush, legendCol, legendRow);
                    PointF pt = new PointF(legendCol + labelSize.Width + s.UlMargin.Width / 2, legendRow);
                    DrawShape(e.Graphics, brush, pt, shpIdx);

                    legendCol += legendWidth;
                    lblIdx++;
                }
            }

            e.Graphics.TranslateTransform(s.UlMargin.Width, s.UlMargin.Height);
            SolidBrush bgGrid = s.BgGrid;
            bgGrid.Color = (this.BackgroundImage != null) ? Color.FromArgb(s.BgGridAlpha, s.BgGrid.Color) : s.BgGrid.Color;
            e.Graphics.FillRectangle(bgGrid, 0, 0, grSize.Width, grSize.Height);

            if (this.dataArray != null)
            {
                Size dotOff = new Size(s.SymSize.Width / 2, s.SymSize.Height / 2);

                for (int dataIdx = 0; dataIdx < this.dataArray.Count; dataIdx++)
                {
                    DataItem di = this.dataArray[dataIdx];

                    // Calculate label Index, Color Index and Shape Index.
                    int lblIdx = labels.IndexOf(di.name);
                    int clrIdx = lblIdx % s.CellBeginColors.Length;
                    int shpIdx = lblIdx / s.CellBeginColors.Length;
                    SolidBrush brush = new SolidBrush(s.CellBeginColors[clrIdx]);

                    // Calculate row and column Point.
                    TimeSpan tspan = di.dt - baseDT;
                    int hourRow = (int)tspan.TotalHours;

                    float row = s.CellSize.Height * hourRow + s.CellSize.Height;
                    float col = ((int)tspan.TotalMinutes % 60) / minInc  * s.CellSize.Width;
                    PointF pt = new PointF(col, row) - dotOff;

                    if (s.ShadowSize.Width > 0 || s.ShadowSize.Height > 0)
                    {
                        PointF shdPt = pt + s.ShadowSize;
                        DrawShape(e.Graphics, s.SymBack, shdPt, shpIdx);
                    }

                    DrawShape(e.Graphics, brush, pt, shpIdx);
                } 
            }

            for (int vColumn = 0; vColumn < grSize.Width; vColumn += s.CellSize.Width)
            {
                e.Graphics.DrawLine(s.VlColor, vColumn, 0, vColumn, grSize.Height);
                e.Graphics.DrawLine(Pens.Black, vColumn + 1, 0, vColumn + 1, grSize.Height);
            }
            for (int hRow = 0; hRow < grSize.Height; hRow += s.CellSize.Height)
            {
                e.Graphics.DrawLine(s.HlColor, 0, hRow, grSize.Width, hRow);
                e.Graphics.DrawLine(Pens.Green, 0, hRow + 1, grSize.Width, hRow + 1);
            }
            
            e.Graphics.DrawRectangle(s.FrameColor, 0, 0, grSize.Width, grSize.Height);
            e.Graphics.TranslateTransform(-s.UlMargin.Width, -s.UlMargin.Height);
        }

        protected override void OnPrint(PaintEventArgs e)
        {
            base.OnPrint(e);
        }

        /// http://www.codeproject.com/KB/printing/printawinformsusercontrol.aspx
        /// <summary>
        /// Print the control's view to a Graphics object.
        /// </summary>
        /// <param name="graphics">Graphics object to draw on.</param>
        /// <param name="bounds">Rectangle to print in.</param>
        public void PrintToGraphics(Graphics graphics, RectangleF bounds)
        {
            Bitmap bitmap = new Bitmap(this.Width, this.Height);
            this.DrawToBitmap(bitmap, new Rectangle(0, 0, bitmap.Width, bitmap.Height));
            RectangleF target = new RectangleF(0, 0, bounds.Width, bounds.Height);
            double xScale = (double)bitmap.Width / bounds.Width;
            double yScale = (double)bitmap.Height / bounds.Height;
            if (xScale < yScale)
                target.Width = (int)(xScale * target.Width / yScale);
            else
                target.Height = (int)(yScale * target.Height / xScale);
            graphics.PageUnit = GraphicsUnit.Display;
            graphics.DrawImage(bitmap, target);
        }

        private void TimeGraph_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.dataArray == null || this.dataArray.Count == 0 || s.CellSize.Width == 0)
                return;

            Size mouseLoc = new Size(e.Location.X, e.Location.Y);
            mouseLoc -= s.UlMargin;
            Size grSize = DisplayRectangle.Size - s.UlMargin - s.LrMargin;

            if (mouseLoc.Width > 0 && mouseLoc.Height > 0)
            {
                int hourOff = ((mouseLoc.Height - s.CellSize.Height/2) / s.CellSize.Height);
                int minOff = (mouseLoc.Width * 60 / grSize.Width);

                DateTime queryDT = baseDT + new TimeSpan(hourOff, minOff, 0);
                // System.Diagnostics.Debug.WriteLine(hourOff.ToString() + " " + minOff.ToString() + " " + queryDT.ToShortTimeString());

                string hitMsg = string.Empty;
                foreach (DataItem di in this.dataArray)
                {
                    if (Math.Abs(di.dt.Hour - queryDT.Hour) <= 0 &&
                        Math.Abs(di.dt.Minute - queryDT.Minute) <= 5)
                    {
                        hitMsg += string.Format("{0} {1} \n", di.name, di.dt.ToString("G"));
                    }
                }

                if (hitMsg.Length > 0)
                {
                    float x = e.Location.X + 20;
                    float y = e.Location.Y + 20;

                    this.toolTip.Show(hitMsg, this, (int)x, (int)y, 5000);
                }
            }
        }
    }
}
