﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WebMetar_NS
{
    /// 
    /// Class for non-numeric Weather parameters.
    /// Supports:
    ///     * Unit enum and string array list (of valid choices)
    ///     * Format value and string array list (of valid choices)
    ///     * Unit conversion (via array of conversion delegates)
    ///     * Text presentation.
    ///     
    /// Author: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// 

    [Serializable]
    class WxString : WxBase
    {
        static string[] defUnitStrs = new string[] { "default" };
        static string[] defFmtStrs = new string[] { "string" };
        static Conversion[] defCnvDelegates = new Conversion[] { DefToUnit };
        static private double DefToUnit(double f, int toUnit)
        {
            return f;
        }

        public WxString() : base(defUnitStrs, defCnvDelegates)
        {
            m_fmtArray = defFmtStrs;
            m_fmtStr = m_fmtArray[0];
            m_unit = 0;
        }

        public override bool IsNumber
        {
            get { return false; }
        }
        public override string Text
        {
            get { return m_text; }
            set { m_text = value; }
        }

        public override string TextandUnit
        {
            get
            {
                return Text;
            }
        }

        string m_text;
    }

    [Serializable]
    class Weather : WxString
    {
        public Weather() 
            : base()
        {
        }

        public override string TextandUnit
        {
            get
            {
                return Text;
            }
        }

        public string Text
        {
            get
            {
                string value = m_proximity + m_intensity + m_descriptor + m_preciptiation + m_obscruation + m_other;
                return value;
            }
        }

        string m_proximity;
        public string Proximity 
        {
            set { m_proximity = value;}
        }

        string m_intensity;
        public string Intensity 
        {
            set { m_intensity = value; }
        }

        string m_descriptor;
        public string Descriptor
        {
            set {m_descriptor = value; }
        }

        string m_preciptiation;
        public string Precipitation
        {
            set {m_preciptiation = value;}
        }

        string m_obscruation;
        public string Obscuration 
        {
            set {m_obscruation = value;}
        }

        string m_other;
        public string Other
        {
            set { m_other = value; }
        }

    }

    [Serializable]
    class Clouds : WxString
    {
        public Clouds() 
            : base()
        {
            m_how = How.Exact;
        }

        public string Cumulus
        {
            set { }
        }

        public LDistance Height = new LDistance();

        public enum How { LessThan, Exact, GreaterThan };
        How m_how;

        public How HowTo
        {
            get { return m_how; }
            set { m_how = value; }
        }

        public override string TextandUnit
        {
            get
            {
                if (Height.IsPopulated)
                {
                    switch (m_how)
                    {
                        case How.LessThan:
                            return base.TextandUnit + " LessThan " + Height.TextandUnit;
                       
                        case How.Exact:
                        default: 
                            return base.TextandUnit + " " + Height.TextandUnit;

                        case How.GreaterThan:
                            return base.TextandUnit + " GreaterThan " + Height.TextandUnit;
                    }
                }
                else
                {
                    return base.TextandUnit;
                }
            }
        }

    }

 
  
}
